                .LF     rz8_reconstruction.lst
;--------------------------------------------------------------------------
;   rz8_reconstruction.ASM
;
;   Attempt to reconstruct a Z8 resident assembler and monitor.
;
; 2020-02-09	Assembles and recreates the known-good hex file.
;		4K data, but duplicated in both halves of an 8K EPROM.
;		Monitor is in the first 2K, the assembler in the next.
;		Partly in BASIC, these lines are in ASCII strings.
;		The machine-code will take considerable analysis,
;		to find what is code or data.
;		I have recreated mostly as code, because it is
;		easier to guess what is not code, e.g.
;		jumps to addresses outside of known ROM space,
;		code that looks nonsensical, etc.
;
;--------------------------------------------------------------------------

                .CR     Z8
;;                .TF     rz8_reconstruction.int,int,32
                .TF     rz8_reconstruction.hex,hex,16
                .LI     TOFF

;--------------------------------------------------------------------------

WORK      	.eq     $74
REG       	.eq     $57
PAIR      	.eq     $36
                .RP     $70             Set register pointer

;--------------------------------------------------------------------------
;   LD INSTRUCTIONS
;--------------------------------------------------------------------------



                .RP     $70             Set register pointer

;
; Equate names must start on left of page.
; Or else you get:
; *** Error: Bad opcode
;
SPACE   .EQ     $20
QUOT    .EQ     $22
COLON	.EQ     $3A

RET_	.EQ	$AF
NOP_	.EQ	$FF
CR	.EQ	$0D
SP	.EQ	$20


P0	.EQ	$00
P1	.EQ	$01
P2	.EQ	$02
P3	.EQ	$03

rr	.eq	$d4

SIO	.EQ	$f0
IRQ	.EQ	$fa
IMR	.EQ	$fb
RP	.EQ	$fd

SPH	.eq	$fe
SPL	.eq	$ff


prog_00007	.EQ	$002b
prog_00012	.EQ	$0054


prog_00008	.EQ	$1615
prog_00010	.EQ	$16e2
prog_00011	.EQ	$1680



	.do	0
L1733	.EQ	$1733
L162c	.EQ	$162c
L1637	.EQ	$1637
L15f0	.EQ	$15f0
L161e	.EQ	$161e
L15db	.EQ	$15db
L15d8	.EQ	$15d8
L1729	.EQ	$1729
L173e	.EQ	$173e

L1641	.EQ	$1641
L169a	.EQ	$169a
L169e	.EQ	$169e
L16ba	.EQ	$16ba
L16d2	.EQ	$16d2

L1654	.EQ	$1654

L15fa	.EQ	$15fa
L1580	.EQ	$1580
L1654	.EQ	$1654
L16be	.EQ	$16be
L16fa	.EQ	$16fa
L1725	.EQ	$1725
	.fi

;
; These are in the Z8 BASIC ROM
;
BASIC_000e:	.eq	$000e
BASIC_02c4:	.eq	$02c4
BASIC_02f8:	.eq	$02f8
BASIC_0556:	.eq	$0556
BASIC_05d3:	.eq	$05d3

; Force these known good:
	.do	0
func_1a04	.eq	$1a04
mark_0183	.eq	$1c7c
mark_0195	.eq	$1bcd
mark_0199	.eq	$1b80
mark_0205	.eq	$1b9f
mark_0210	.eq	$1cf5
mark_0232	.eq	$1b97
mark_0270	.eq	$1a08
prog_00017	.eq	$19c3
prog_00019	.eq	$1a24
prog_00020	.eq	$1a3f
prog_00021	.eq	$1984
prog_00022	.eq	$1a97
prog_00023	.eq	$19f9
prog_00024	.eq	$1be9
prog_00025	.eq	$176d
prog_00026	.eq	$176a
prog_00027	.eq	$1764
prog_00028	.eq	$1788
prog_00029	.eq	$1fc7
prog_00030	.eq	$1761
prog_00031	.eq	$175e

prog_00033	.eq	$1b8e
prog_00034	.eq	$1e39
	.fi


;	EPROM was marked
;	RZ8 40 V4.1
;
;	This is a Resident Z8 Assembler for the Arcom ARC40 board
;
;	The 4K ROM resides at $1000 to $1FFF, so:

	.org	$1000



;	1000	4096	Jump to routine for IRQ0.
	.db	$FF,$FF,$FF
;	1003	4099	Jump to routine for IRQ1.
	.db	$FF,$FF,$FF
;	1006	4102	Jump to routine for IRQ2.
	.db	$FF,$FF,$FF
;	1009	4105	Jump to routine for IRQ3.
	.db	$FF,$FF,$FF
;	100C	4108	Jump to routine for IRQ4.
	.db	$FF,$FF,$FF
;	100F	4111	Address for IRQ5. When an interrupt occurs at IRQ5, the Z8671 vectors through internal ROM to 100F to get a jump to the address of a user-supplied processing routine.
	.db	$FF,$FF,$FF


;	1012	4114	Address for external input driver. Store a jump to a user-supplied output driver here.
	.db	$8D,$17,$33

;	1015	4117	Address for external output driver. Store a jump to a user-supplied output driver here.
	.db	$AF,$FF,$FF

	.db	$FF,$FF,$FF
	.db	$FF,$FF,$FF
	.db	$FF,$FF

;	1020	4128	First address of Basic/Debug program for auto-start up. Store program starting in byte 20.

;
; from 1020 to  1550 hex:
;
	.dw	1					;	1 = line number, msb first.

;
; You can use / to enclose strings with many quotation marks:
;
	.az	/"":"ARC40 RZ8 V4.1 ":"":"'1' for new program"/
;
; but of course this:
;
;	.az	/a over b = a//b/
;
; gets *** Error: Bad Operand
;
;

	.dw	2
	.az	/"'2' for old program":"'3' to copy monitor":"'4' to run in EPROM":""/
	.dw	3
	.az	/inputa:ifa<1goto2/
	.dw	4
	.az	/ifa>4goto2/
	.dw	5
	.az	/ifa=4stop/
	.dw	7
	.az	/@36=a:go@%1770/
	.dw	8
	.az	/stop/

	.dw	30030
	.az	/^28=%17f0/
L10E8:
	.dw	30031
	.az	/"File no.";:n=usr(84):" "/
	.dw	30032
	.az	/"Start recorder, hit key";:z=usr(84):" "/
L112F:
	.dw	30033
	.az	/go@97,36:go@97,n:list/
	.dw	30034
	.az	/^28=39:stop/

	.dw	30040
	.az	/"File no. ";:n=usr(84):" "/
	.dw	30041
	.az	/go@%1773,n:stop/
	.dw	30060
	.az	/gosub30095/
	.dw	30061
	.az	/"RAM start addr. ";:ina:"End addr. ";:inb:"EPROM addr. ";:ins/
	.dw	30062
	.az	/e=s+b-a+1:ife>n*1024goto30088/
L11F1:
	.dw	30063
	.az	/x=%f800:z=%fa00:@63=r:ifusr(%1776,s,e)=0goto30070/
	.dw	30064
	.az	/"%";hex(^50),"%";hex(@x),"not %FF. Enter C to continue ";/
	.dw	30065
	.az	/ifand(usr(84),%5f)<>%43goto30089/
	.dw	30066
	.az	/"":ifusr(%1779)<>0goto30064/

	.dw	30070
	.az	/^54=a:@3=%20:@%fb00=%80:@z=p-128:@z=p:@63=p/
	.dw	30071
	.az	/go@%178e,s,e/
	.dw	30080
	.az	/^54=a:@%fb00=%90:@63=r:@3=0:ifusr(%177c,s,e)=0goto30087/
	.dw	30082
	.az	/"%";hex(^50),"%";hex(@x),"not %";hex(@^54)/
	.dw	30083
	.az	/ifusr(%177f)<>0goto30082/

	.dw	30087
	.az	/@z=0:"":"OK":stop/
L1375:
	.dw	30088
	.az	/"":"> EPROM size"/

	.dw	30089
	.az	/@z=0:"":"Aborted":stop/
	.dw	30090
	.az	/gosub30095/
	.dw	30091
	.az	/^8=%1020:goto30092/
	.dw	30092
	.az	/"EPROM start addr. ";:ins:"End addr. ";:ine:"RAM addr. ";:ina/
	.dw	30093
	.az	/@3=0:@63=r:^54=a:go@%1785,s,e+1:"":"OK":goto1/
L1434:
	.dw	30095
	.az	/"":"Select 2716 (0), 2732 (1), 2764 (2) or 27128 (3) : ";:ing/
	.dw	30096
	.az	/ifg>3goto30095/
	.dw	30097
	.az	/r=%40:ifg>1r=0/
	.dw	30098
	.az	/p=%80:ifg=0p=%c0/
	.dw	30099
	.az	/n=16:ifg<3n=8:ifg<2n=4:ifg<1n=2/
	.dw	30100
	.az	/"":"";n;"k EPROM":"":ret/
L14E6:
	.dw	30200
	.az	/"Address ";:ina/
L14F8:
	.dw	30201
	.az	/n=@a:"%";hex(a),"%";hex(n),:ind:@a=d:a=a+1:goto30201/
L152F:
	.dw	30210
	.az	/"Address ";:ina:go@%178b,a:stop/

L1551:
	.db	$FF,$FF,$FF,$FF,$FF,$FF,$FF
;	1558
	.db	$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
;	1560
	.db	$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
;	1568
	.db	$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
;	1570
	.db	$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
;	1578
	.db	$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF


L1580:

	.do 0
;	1580
	.db	$31,$30,$E8,$14,$F8,$15,$C8,$EE	;	db "10"
;	1588
	.db	$D8,$EF,$D6,$15,$F0,$3C,$25,$D6
;	1590
	.db	$16,$2C,$68,$EC,$D6,$15,$DB,$68
;	1598
	.db	$ED,$D6,$15,$DB,$C2,$6C,$D6,$15
;	15A0
	.db	$D8,$A0,$EC,$76,$ED,$0F,$6B,$0A
;	15A8
	.db	$76,$ED,$07,$EB,$EF,$D6,$16,$37
;	15B0
	.db	$8B,$EA,$D6,$16,$37,$C2,$3E,$A6
;	15B8
	.db	$E3,$20,$7B,$05,$A6,$E3,$7F,$7B
;	15C0
	.db	$02,$3C,$2E,$D6,$16,$2C,$A0,$EE
;	15C8
	.db	$76,$EF,$0F,$EB,$E8,$D6,$15,$FA
;	15D0
	.db	$EB,$B8,$D6,$15,$F0,$31,$10,RET_
L15d8:
;	15D8
	.db	$D6,$16,$37,$69,$44,$70,$FD,$31
;	15E0
	.db	$40,$38,$E4,$F0,$E3,$D6,$16,$1E
;	15E8
	.db	$38,$E4,$D6,$16,$1E,$50,$FD,RET_
;	15F0
	.db	$70,$FD,$31,$10,$D6,$00,$2B,$50
;	15F8
	.db	$FD,RET_,$B0,$E3,$76,$FA,$08,$6B
;	1600
	.db	$10,$D6,$16,$15,$A6,$E3,$13,$EB
;	1608
	.db	$08,$76,$FA,$08,$6B,$FB,$D6,$16
;	1610
	.db	$15,$A6,$E3,$1B,RET_,$38,$F0,$56
;	1618
	.db	$E3,$7F,$56,$FA,$F7,RET_,$56,$E3
;	1620
	.db	$0F,$A6,$E3,$0A,$7B,$03,$06,$E3
;	1628
	.db	$07,$06,$E3,$30,$56,$FA,$EF,$39
;	1630
	.db	$F0,$76,$FA,$10,$6B,$FB,RET_,$70
;	1638
	.db	$E3,$3C,$20,$D6,$16,$2C,$50,$E3
;	1640
	.db	RET_

	.db	$D6,$16,$D2,$80,$E2,$A0,$E2
;	1648
	.db	$D6,$16,$E2,$6B,$41,$C2,$EC,$EE
;	1650
	.db	$6B,$F4,$8B,$41,$D6,$16,$80,$D6
;	1658
	.db	$16,$E2,$6B,$1E,$C2,$06,$A0,$E6
;	1660
	.db	$D2,$0C,$CC,$FA,$B6,$EE,$40,$D2
;	1668
	.db	$EC,$8C,$1A,$B0,$E9,$80,$E8,$EB
;	1670
	.db	$FC,$B6,$EE,$40,$D2,$EC,$A0,$E2
;	1678
	.db	$8B,$DD,$56,$EE,$40,$D2,$EC,RET_
;	1680
	.db	$31,$30,$28,$14,$38,$15,$48,$12
;	1688
	.db	$58,$13,$56,$E2,$3F,RET_

	.db	$31,$10
;	1690
	.db	$B0,$E2,$B0,$E3,RET_

	.db	$31,$10,$3C
;	1698
	.db	NOP_,RET_

	.db	$31,$30,$8B,$A8,$D6,$16
;	16A0
	.db	$D2,$80,$E6,$80,$E2,$A0,$E6,$A0
;	16A8
	.db	$E2,$D6,$16,$E2,$6B,$E0,$C2,$EC
;	16B0
	.db	$CE,$CE,$C2,$06,$A2,$0E,$6B,$ED
;	16B8
	.db	$8B,$DB,$31,$30,$8B,$E7,$31,$30
;	16C0
	.db	$D6,$16,$D2,$D6,$16,$E2,$6B,$C6
;	16C8
	.db	$C2,$0C,$D2,$06,$A0,$E6,$A0,$E2
;	16D0
	.db	$8B,$F1,$D6,$16,$80,$CC,$FB,$B0
;	16D8
	.db	$ED,$0C,$90,$D2,$0C,$00,$EC,$D2
;	16E0
	.db	$FC,RET_

	.db	$CC,$FA,$E8,$E2,$56,$EE
;	16E8
	.db	$3F,$42,$EF,$D2,$EC,$00,$EC,$D2
;	16F0
	.db	$3C,$00,$EC,$A2,$35,$EB,$02,$A2
;	16F8
	.db	$24,RET_

	.db	$0C,$00,$D2,$02,$A0,$E2
;	1700
	.db	$0E,$D2,$02,$80,$E2,$1C,NOP_,$C2
;	1708
	.db	$04,$D2,$02,$A0,$E2,$A0,$E4,$A2
;	1710
	.db	$01,$EB,$F4,$C2,$04,$D2,$02,$A2
;	1718
	.db	$01,$EB,$EC,$06,$E3,$1F,$16,$E2
;	1720
	.db	$00,$29,$04,$39,$05,$E6,$FB,$80
;	1728
	.db	RET_

	.db	$E6,$22,$17,$E6,$23,$A0,$E6
;	1730
	.db	$FB,$88,RET_

	.db	$31,$20,$C2,$12,$A0
;	1738
	.db	$E2,$31,$10,$38,$21,RET_

	.db	$D6,$00
;	1740
	.db	$54,$A6,$E3,$24,$EB,$F8,$D6,$00
;	1748
	.db	$54,$22,$35,$EB,$F1,RET_


	.el

L1580:
	srp    #48
	ld     r14,20
	ld     r15,21
	ld     r12,r14
	ld     r13,r15

mark_0074:

L158a:	call	L15f0
	ld	r3,#37
L158f:	call	L162c
	ld	r6,r12
	call	L15db
	ld	r6,r13
	call	L15db

mark_0070:

L159c:	ldc	r6,@rr12
	call	L15d8
	incw	rr12
	tm	r13,#15
	jr	z,mark_0069
	tm	r13,#7
	jr	nz,mark_0070
	call	L1637
	jr	mark_0070

mark_0069:

L15b2:	call	L1637

mark_0073:

L15b5:	ldc	r3,@rr14
	cp	r3,#32
	jr	c,mark_0071
	cp	r3,#127
	jr	c,mark_0072

mark_0071:

L15c1:	ld	r3,#46

mark_0072:

L15c3:	call	L162c
	incw	rr14
	tm	r15,#15
	jr	nz,mark_0073
	call	L15FA
	jr	nz,mark_0074
	call	L15f0

	srp	#16

	ret

L15d8:
	call	L1637	;	15d8  d6 16 37
L15db:
	ld	68,r6
	push	RP
	srp	#64
	ld	r3,r4
	swap	r3
	call	L161e
	ld	r3,r4
	call	L161e
	pop	RP
	ret
L15f0:
	push	RP	;	15f0  70 fd
	srp	#16
	call	prog_00007
	pop	RP
	ret
L15fa:
	clr	r3	;	15fa  b0 e3
	tm	IRQ,#8
	jr	z,mark_0075
	call	prog_00008
	cp	r3,#19
	jr	nz,mark_0075
mark_0076:	;
	tm	IRQ,#8	;	1609  76 fa 08
	jr	z,mark_0076
	call	prog_00008
mark_0075:	;
	cp	r3,#27	;	1611  a6 e3 1b
	ret

prog_00008:
	ld	r3,SIO
	and	r3,#127
	and	IRQ,#247
	ret

L161e:
	and	r3,#15
	cp	r3,#10
	jr	c,mark_0077
	add	r3,#7
mark_0077:	;
	add	r3,#48	;	1629  06 e3 30
L162c:
	and	IRQ,#239
	ld	SIO,r3
mark_0078:
	tm	IRQ,#16	;	1631  76 fa 10
	jr	z,mark_0078
	ret


L1637:
	push	r3
	ld	r3,#32
	call	L162c
	pop	r3
	ret
L1641:
	call	L16D2
	decw	rr2
mark_0080:	;
	incw	rr2	;	1646  a0 e2
	call	prog_00010
	jr	z,mark_0079
	ldc	r14,@rr12
	inc	r14
	jr	z,mark_0080
	jr	mark_0081

L1654:
	call	prog_00011
mark_0084:	;
	call	prog_00010	;	1657  d6 16 e2
	jr	z,mark_0082
	ldc	r0,@rr6
	incw	rr6
	ldc	@rr12,r0
	ld	r12,#250
	xor	r14,#64
	ldc	@rr12,r14
	ld	r8,#26
	clr	r9
mark_0083:	;
	decw	rr8	;	166d  80 e8
	jr	nz,mark_0083
	xor	r14,#64
	ldc	@rr12,r14
	incw	rr2
	jr	mark_0084
mark_0082:	;
	and	r14,#64	;	167a  56 ee 40
	ldc	@rr12,r14
	ret
prog_00011:

	srp	#48
	ld	r2,20
	ld	r3,21
	ld	r4,18
	ld	r5,19
	and	r2,#63
	ret
mark_0079:	;
	srp	#16	;	168e  31 10
	clr	r2
	clr	r3
	ret
mark_0081:	;
	srp	#16	;	1695  31 10
	ld	r3,#255
	ret

L169a:
	srp	#48
	jr	mark_0080

L169e:
	call	L16D2	;; not L16be
	decw	rr6
	decw	rr2
L16a5:	;
	incw	rr6	;	16a5  a0 e6
	incw	rr2
	call	prog_00010
	jr	z,mark_0079
	ldc	r14,@rr12
	inc	r12
	inc	r12
	ldc	r0,@rr6
	cp	r0,r14
	jr	z,L16a5
	jr	mark_0081

L16ba:

	srp	#48
	jr	L16a5

L16be:

	srp	#48
	call	L16d2
mark_0086:	;
	call	prog_00010	;	16c3  d6 16 e2
	jr	z,mark_0079
	ldc	r0,@rr12
	ldc	@rr6,r0
	incw	rr6
	incw	rr2
	jr	mark_0086

L16d2:
	call	prog_00011
	ld	r12,#251
	clr	r13
	ld	r0,#144
	ldc	@rr12,r0
	dec	r12
	ldc	@rr12,r15
	ret

prog_00010:
	ld	r12,#250	;	16e2  cc fa
	ld	r14,r2
	and	r14,#63
	or	r14,r15
	ldc	@rr12,r14
	dec	r12
	ldc	@rr12,r3
	dec	r12
	cp	r3,r5
	jr	nz,mark_0087
	cp	r2,r4
mark_0087:	;
	ret		;	16f9  af

L16fa:
	ld	r0,#0	;	16fa  0c 00
	ldc	@rr2,r0
	incw	rr2
	inc	r0
	ldc	@rr2,r0
	decw	rr2
	ld	r1,#255
mark_0088:
	ldc	r0,@rr4	;	1707  c2 04
	ldc	@rr2,r0
	incw	rr2
	incw	rr4
	cp	r0,r1
	jr	nz,mark_0088

	ldc	r0,@rr4
	ldc	@rr2,r0
	cp	r0,r1
	jr	nz,mark_0088
	add	r3,#31
	adc	r2,#0
	ld	4,r2
	ld	5,r3

L1725:
	ld	IMR,#128
	ret

L1729:
	ld	34,#23	;	1729  e6 22 17
	ld	35,#160
	ld	IMR,#136
	ret

L1733:
	srp	#32	;	1733  31 20
	ldc	r1,@rr2
	incw	rr2
	srp	#16
	ld	r3,33
	ret

L173e:
	call	prog_00012	;	173e  d6 00 54
	cp	r3,#36
	jr	nz,L173e
	call	prog_00012
	sub	r3,r5
	jr	nz,L173e
	ret

	.fi

;	174E

	.do	0

	.db	NOP_
	.db	NOP_, NOP_, NOP_
	.db	NOP_, NOP_, NOP_
	.db	NOP_, NOP_, NOP_

;	174e  ff      		nop
;
;	174f  ff ff ff		nop
;	1752  ff ff ff		nop
;	1755  ff ff ff		nop
;	1758  ff ff ff		nop
;	175b  ff ff ff		nop


;	175e  8d 16 2c		jp     L162c
;	1761  8d 16 37		jp     L1637
;	1764  8d 15 f0		jp     L15f0
;	1767  8d 16 1e		jp     L161e
;	176a  8d 15 db		jp     L15db
;	176d  8d 15 d8		jp     L15d8
;	1770  8d 17 29		jp     L1729
;	1773  8d 17 3e		jp     L173e
;	1776  8d 16 41		jp     L1641	; entry point
;	1779  8d 16 9a		jp     L169a	; entry point
;	177c  8d 16 9e		jp     L169e	; entry point
;	177f  8d 16 ba		jp     L16ba	; entry point
;	1782  8d 16 d2		jp     L16d2
;	1785  8d 16 be		jp     L16be
;	1788  8d 15 fa		jp     L15fa
;	178b  8d 15 80		jp     L1580
;	178e  8d 16 54		jp     L1654
;	1791  8d 16 fa		jp     L16fa
;	1794  8d 17 25		jp     L1725
;	1797  8d 17 33		jp     L1733
;	179a  ff ff ff		nop
;	179d  ff ff ff		nop





;	1758
	.db	NOP_,NOP_,NOP_
	.db	NOP_,NOP_,NOP_
	.db	$8D,$16,$2C
;	1761
	.db	$8D,$16,$37
	.db	$8D,$15,$F0
	.db	$8D
;	1768
	.db	$16,$1E,$8D,$15,$DB,$8D,$15,$D8
;	1770
	.db	$8D,$17,$29
	.db	$8D,$17,$3E
	.db	$8D,$16,$41
	.db	$8D,$16,$9A
	.db	$8D,$16,$9E
	.db	$8D,$16,$BA
	.db	$8D,$16,$D2
	.db	$8D,$16,$BE
;	1788
	.db	$8D,$15,$FA
	.db	$8D,$15,$80
	.db	$8D,$16,$54
	.db	$8D,$16,$FA
	.db	$8D,$17,$25
	.db	$8D,$17,$33
	.db	NOP_,NOP_,NOP_
	.db	NOP_,NOP_,NOP_

	.el

	nop

	.db	NOP_,NOP_,NOP_
	.db	NOP_,NOP_,NOP_
	.db	NOP_,NOP_,NOP_
	.db	NOP_,NOP_,NOP_
	.db	NOP_,NOP_,NOP_
prog_00031:	;	$175e
	jp     L162c
prog_00030:	;	$1761
	jp     L1637
prog_00027:	;	$1764
	jp     L15f0
	jp     L161e
prog_00026:	;	$176a
	jp     L15db
prog_00025:	;	$176d
	jp     L15d8
	jp     L1729
	jp     L173e
	jp     L1641	; entry point
	jp     L169a	; entry point
	jp     L169e	; entry point
	jp     L16ba	; entry point
	jp     L16d2
	jp     L16be
prog_00028:	;	$1788
	jp     L15fa
	jp     L1580
	jp     L1654
	jp     L16fa
	jp     L1725
	jp     L1733



	.db	NOP_,NOP_,NOP_
	.db	NOP_,NOP_,NOP_

	.fi






;	17A0
;
; More BASIC
;
;	from 17a0 to  17e7./
;	No line numbers!/
;	Maybe commands to execute immediately./

	.as	/^8=%4000:^12=^10:new/
	.db	CR
	.as	/s=%1020/
	.db	CR
	.as	/d=^8/
	.db	CR
	.as	/ifa=1go@%1794/
	.db	CR
	.as	/ifa=2s=d/
	.db	CR
	.as	/go@%1791,s,d/
	.db	CR

	.db	NOP_,NOP_
	.db	NOP_,NOP_,NOP_,NOP_,NOP_,NOP_,NOP_,NOP_
;	17F0
	.db	$06,$50,$20,$08,$1B,$0D,$0A,$00
;	17F8
	.db	$00,$00,$00,$00,$00,$00,$00,$FE
;	1800
	.db	$B0,$55,$8D,$1C,$03,$E6,$55,NOP_	;	.U....U.
;	1808
	.db	$8B,$F8
	.db	"ADC"
	.db	$01,$10
	.db	"ADD"
	.db	$01,$00
	.db	"AND"
	.db	$01,$50
	.db	"CALL"
	.db	$05,$D4
	.db	"CCF"
	.db	$07,$EF
	.db	"CLR"
	.db	$0B,$B0
	.db	"COM"
	.db	$0B,$60
	.db	"CP"
	.db	$01,$A0
	.db	"DA"
	.db	$0B,$40
	.db	"DECW"
	.db	$0C,$80
	.db	"DEC"
	.db	$0B,$00
	.db	"DI"
	.db	$07,$8F
	.db	"DJNZ"
	.db	$08,$0A
	.db	"EI"
	.db	$07,$9F
	.db	"INCW"
	.db	$0C,$A0
	.db	"INC"
	.db	$0D,$20
	.db	"IRET"
	.db	$07,$BF
	.db	"JP"
	.db	$06,$0D
	.db	"JR"
	.db	$09,$0B
	.db	"LDCI"
	.db	$03,$C3
	.db	"LDC"
	.db	$02,$C2
	.db	"LDEI"
	.db	$03,$83
	.db	"LDE"
	.db	$02,$82
	.db	"LD"
	.db	$0A,$00
	.db	"NOP"
	.db	$07,NOP_
	.db	"OR"
	.db	$01,$40
	.db	"POP"
	.db	$0B,$50
	.db	"PUSH"
	.db	$0B,$70
	.db	"RCF"
	.db	$07,$CF
	.db	"RET"
	.db	$07,$AF
	.db	"SBC"
	.db	$01,$30
	.db	"SCF"
	.db	$07,$DF
	.db	"RLC"
	.db	$0B,$10
	.db	"RL"
	.db	$0B,$90
	.db	"RRC"
	.db	$0B,$C0

	.do	0
	.db	"RC"	; 52 43
	.el
	.db	"RR"	; 52 52	according to known good ROM
	.fi


	.db	$0B,$E0
	.db	"SRA"
	.db	$0B,$D0
	.db	"SRP"
	.db	$04,$31
	.db	"SUB"
	.db	$01,$20
	.db	"SWAP"
	.db	$0B,$F0
	.db	"TCM"
	.db	$01,$60
	.db	"TM"
	.db	$01,$70
	.db	"XOR"
	.db	$01,$B0
	.db	"END"
	.db	$0F,$00
	.db	"$ABS"
	.db	$0E,$02
	.db	"DEFS"
	.db	$0E,$03
	.db	"DEFB"
	.db	$0E,$04
	.db	$FF
	.db	"C"
	.db	$07
	.db	"NC"
	.db	$0F
	.db	"Z"
	.db	$06
	.db	"NZ"
	.db	$0E
	.db	"PL"
	.db	$0D
	.db	"MI"
	.db	$05
	.db	"OV"
	.db	$04
	.db	"NOV"
	.db	$0C
	.db	"EQ"
	.db	$06
	.db	"NE"
	.db	$0E
	.db	"GE"
	.db	$09
	.db	"LT"
	.db	$01
	.db	"GT"
	.db	$0A
	.db	"LE"
	.db	$02
	.db	"UGE"
	.db	$0F
	.db	"ULT"
	.db	$07
	.db	"UGT"
	.db	$0B
	.db	"ULE"

	.do	0


	.db	$03


;	1930
	.db	$FF,$1C,$E3,$1D,$46,$1D,$57,$1D
;	1938
	.db	$6B,$1D,$73,$1D,$9B,$1D,$DF,$1D
;	1940
	.db	$E4,$1D,$FC,$1E,$44,$1E,$FA,$1F
;	1948
	.db	$15,$1F,$34,$1F,$41,$1F,$A9,$27
;	1950
	.db	$10,$03,$E8,$00,$64,$00,$0A,$00
;	1958
	.db	$01,$C2,$1A,$A0,$EA,RET_

	.el
;
; If this is code, I think it got mangled.
; Is 03 data from the previous table?
; FF might be the terminator.
;
; I think this is code - some of the jp jump to infeasible addresses.
;
	add	r15,@r15	;	192f  03 ff

	ld	r1,#227		;	1931  1c e3

;;	jp	lt,mark_0132	;	1933  1d 46 1d
	.db	$1d,$46,$1d


	and	@29,#107	;	1936  57 1d 6b
;;	jp	lt,mark_0133	;	1939  1d 73 1d
	.db	$1d,$73,$1d


	jr	ge,mark_0134	;	193c  9b 1d

	scf			;	193e  df

;;	jp	lt,mark_0135	;	193f  1d e4 1d
	.db	$1d,$e4,$1d

	ld	r15,#30		;	1942  fc 1e
	or	IRQ,30		;	1944  44 1e fa
	.db	$1f		;	1947  1f

;	adc	@52,31		;	1948  15 1f 34	sbasm complains bad operand

	.db	$15, $1f, $34

	.db	$1f		;	194b  1f
	da	@31		;	194c  41 1f
	ld	39,r10		;	194e  a9 27
	rlc	P3		;	1950  10 03
	ld	r14,P0		;	1952  e8 00
	tcm	10,P0		;	1954  64 00 0a
	dec	P1		;	1957  00 01
	ldc	r1,@rr10	;	1959  c2 1a
mark_0134:
	incw	rr10		;	195b  a0 ea
	ret			;	195d  af

	.fi


;	195E-20             1112         .db     SP

	.db	SP
;	195F	
	.db	"No Erro"
	.db	$F2
	.db	"Copyright 1982 (C) Arcom Ltd"
	.db	$AE	; '.'+80h ?

	.do	0

	.db	$68,$EA,$78,$EB
;	1988
	.db	$C8,$E1,$A8,$E6,$B8,$E7,$18,$EC
;	1990
	.db	$C2,$24,$A0,$E4,$A6,$E2,$0F,$BB
;	1998
	.db	$02,$DF,RET_


	.db	$A6,$E1,$41,$7B,$03
;	19A0
	.db	$56,$E1,$DF,$A2,$12,$6B,$18,$CF
;	19A8
	.db	$C2,$24,$A0,$E4,$A6,$E2,$0F,$BB
;	19B0
	.db	$F7,$F0,$E0,$EB,$02,$A0,$E4,$C2
;	19B8
	.db	$24,$A6,$E2,$FF,$EB,$CC,RET_
	.db	$D4
;	19C0
	.db	$2A,$8B,$CD,$70,$FD,$A9,$1A,$B9
;	19C8
	.db	$1B,$A6,$E1,$25,$E4,$2E,$FD,$EB
;	19D0
	.db	$0D,$B9,$57,$D6,$02,$F8,$A4,$EB
;	19D8
	.db	$57,$6D,$1A,$04,$8B,$0B,$80,$EA
;	19E0
	.db	$D6,$05,$D3,$9D,$1A,$04,$D6,$02
;	19E8
	.db	$C4,$D6,$00,$0E,$50,$FD,$A8,$1A
;	19F0
	.db	$B8,$1B,$58,$12,$68,$13,$D4,$2A
;	19F8
	.db	RET_
	.db	$D6,$19,$C3,$42,$55,$ED,$1A
;	1A00
	.db	$04,$58,$E6,RET_
	.db	$31,$30,$9C,$61
;	1A08
	.db	$8C,$19,$D6,$05,$56,$E4,$23,$FE
;	1A10
	.db	$E4,$24,$FF,$E4,$2E,$FD,$D6,$00
;	1A18
	.db	$2B,$E4,$51,$12,$E4,$52,$13,$A8
;	1A20
	.db	$21,$B8,$22,RET_
	.db	$A6,$E1,$20,$6B
;	1A28
	.db	$15,$A6,$E1,$2C,$6B,$10,$A6,$E1
;	1A30
	.db	$09,$6B,$0B,$A6,$E1,$21,$6B,$05
;	1A38
	.db	$A6,$E1,$00,$EB,$01,$DF,RET_
	.db	$D6
;	1A40
	.db	$1A,$24,$EB,$02,$FB,$01,RET_
	.db	$D4
;	1A48
	.db	$2A,$8B,$F4,$D6,$1A,$3F,$EB,$05
;	1A50
	.db	$FB,$03,$7C,$88,RET_
	.db	$A6,$E1,$22
;	1A58
	.db	$EB,$03,$7C,$0A,RET_
	.db	$A6,$E1,$26
;	1A60
	.db	$EB,$03,$7C,$0B,RET_
	.db	$4C,$18,$5C
;	1A68
	.db	$F7,$0C,$FF,$D6,$19,$84,$FB,$09
;	1A70
	.db	$F0,$E2,$42,$32,$D6,$1A,$3F,$8B
;	1A78
	.db	$06,$A8,$E6,$B8,$E7,$18,$EC,$B0
;	1A80
	.db	$E7,$D6,$1A,$97,$48,$E5,$F0,$E7
;	1A88
	.db	$D6,$1A,$3F,$EB,$06,$FB,$04,$46
;	1A90
	.db	$E7,$08,RET_
	.db	$D6,$1A,$97,RET_
	.db	$A6
;	1A98
	.db	$E1,$52,$6B,$05,$A6,$E1,$72,$EB
;	1AA0
	.db	$21,$D4,$2A,$56,$E7,$F0,$A6,$E1
;	1AA8
	.db	$52,$6B,$05,$A6,$E1,$72,$EB,$05
;	1AB0
	.db	$D4,$2A,$46,$E7,$06,$D6,$19,$F9
;	1AB8
	.db	$A6,$E5,$10,$FD,$1A,$04,$46,$E5
;	1AC0
	.db	$E0,RET_
	.db	$A6,$E1,$40,$EB,$29,$D4
;	1AC8
	.db	$2A,$A6,$E1,$52,$6B,$05,$A6,$E1
;	1AD0
	.db	$72,$EB,$16,$D4,$2A,$46,$E7,$01
;	1AD8
	.db	$A6,$E1,$52,$6B,$05,$A6,$E1,$72
;	1AE0
	.db	$EB,$D3,$46,$E7,$07,$D4,$2A,$8B
;	1AE8
	.db	$CC,$46,$E7,$03,$D6,$19,$F9,RET_
;	1AF0
	.db	$A6,$E1,$23,$EB,$07,$D4,$2A,$46
;	1AF8
	.db	$E7,$04,$8B,$F0,$A6,$E1,$2A,$EB
;	1B00
	.db	$3C,$46,$E7,$0F,$F0,$2F,$6B,$07
;	1B08
	.db	$D4,$2A,$F0,$E1,$EB,$FA,RET_
	.db	$D4
;	1B10
	.db	$2A,$D6,$19,$C3,$59,$2C,$69,$2D
;	1B18
	.db	$E6,$2F,$FF,$31,$40,$18,$27,$A8
;	1B20
	.db	$28,$B8,$29,$E8,$04,$F8,$05,$80
;	1B28
	.db	$EA,$C2,$1A,$E4,$51,$53,$E4,$52
;	1B30
	.db	$54,$D6,$1B,$E9,$E9,$35,$F9,$36
;	1B38
	.db	$B0,$2F,$31,$30,RET_
	.db	$D6,$19,$C3
;	1B40
	.db	$A6,$E1,$28,$EB,$2C,$A6,$E5,$00
;	1B48
	.db	$ED,$1A,$04,$D4,$2A,$69,$50,$A6
;	1B50
	.db	$E1,$52,$6B,$06,$A6,$E1,$72,$ED
;	1B58
	.db	$1A,$04,$46,$E7,$05,$D4,$2A,$D6
;	1B60
	.db	$19,$F9,$A6,$E5,$10,$FD,$1A,$04
;	1B68
	.db	$A6,$E1,$29,$ED,$1A,$04,$D4,$2A
;	1B70
	.db	RET_
	.db	$A6,$E5,$00,$6B,$04,$46,$E7
;	1B78
	.db	$0E,RET_
	.db	$46,$E7,$02,$58,$E6,RET_
;	1B80
	.db	$56,$E4,$0F,$F0,$E4,$56,$E5,$0F
;	1B88
	.db	$42,$45,$8C,$02,$8B,$11,$68,$E4
;	1B90
	.db	$48,$E5,$58,$E6,$8C,$03,RET_
	.db	$F0
;	1B98
	.db	$E4,$38,$E4,$48,$E5,$8C,$02,$F0
;	1BA0
	.db	$2F,$6B,$07,$02,$F8,$16,$EE,$00
;	1BA8
	.db	$8B,$23,$2C,$33,$F0,$55,$EB,$0A
;	1BB0
	.db	$68,$EE,$D6,$17,$6D,$68,$EF,$D6
;	1BB8
	.db	$17,$6A,$E3,$62,$04,$56,$EE,$D3
;	1BC0
	.db	$2E,$24,$56,$EE,$F0,$55,$EB,$03
;	1BC8
	.db	$D6,$17,$6D,$8A,$ED,$A6,$E1,$00
;	1BD0
	.db	$6B,$04,$D4,$2A,$8B,$F7,$D4,$2A
;	1BD8
	.db	$A6,$E1,$FF,$19,$51,$C2,$1A,$19
;	1BE0
	.db	$52,$EB,$06,$A6,$E1,$FF,$6D,$1A
;	1BE8
	.db	$04,$F0,$2F,$6B,$11,$A4,$E1,$2D
;	1BF0
	.db	$EB,$0C,$80,$EA,$C2,$1A,$A4,$E1
;	1BF8
	.db	$2C,$EB,$01,RET_
	.db	$A0,$EA,$A0,$EA
;	1C00
	.db	$8D,$1C,$7C,$E4,$14,$51,$E4,$15
;	1C08
	.db	$52,$E6,$2A,$19,$E6,$2B,$59,$E4
;	1C10
	.db	$1A,$21,$E4,$1B,$22,$E4,$FD,$2E
;	1C18
	.db	$E4,$FE,$23,$E4,$FF,$24,$31,$30
;	1C20
	.db	$B0,$56,$A4,$14,$12,$EB,$05,$A4
;	1C28
	.db	$15,$13,$6B,$03,$E4,$12,$56,$F0
;	1C30
	.db	$55,$EB,$0D,$D6,$17,$64,$8C,$19
;	1C38
	.db	$9C,$67,$D6,$05,$56,$D6,$17,$64
;	1C40
	.db	$A8,$08,$B8,$09,$E8,$04,$F8,$05
;	1C48
	.db	$B0,$E1,$A6,$E1,$00,$EB,$0E,$D4
;	1C50
	.db	$2A,$A4,$14,$E1,$EB,$07,$D4,$2A
;	1C58
	.db	$A4,$15,$E1,$6B,$11,$A6,$E1,$FF
;	1C60
	.db	$EB,$08,$D4,$2A,$A6,$E1,$FF,$6D
;	1C68
	.db	$1A,$04,$D4,$2A,$8B,$DC,$E4,$FE
;	1C70
	.db	$25,$E4,$FF,$26,$19,$27,$A9,$28
;	1C78
	.db	$B9,$29,$B0,$2F,$F0,$2F,$EB,$38
;	1C80
	.db	$F0,$55,$EB,$2C,$31,$40,$D6,$17
;	1C88
	.db	$88,$6D,$1A,$04,$D6,$17,$64,$08
;	1C90
	.db	$51,$18,$52,$D6,$1F,$C7,$D6,$17
;	1C98
	.db	$61,$EC,$19,$48,$3A,$58,$3B,$C2
;	1CA0
	.db	$34,$52,$33,$EB,$04,$80,$E4,$3C
;	1CA8
	.db	$20,$A0,$E4,$D6,$17,$5E,$EA,$EF
;	1CB0
	.db	$31,$30,$E4,$25,$FE,$E4,$26,$FF
;	1CB8
	.db	$D4,$2A,$B0,$E0,$D6,$1A,$3F,$6D
;	1CC0
	.db	$1B,$CD,$4C,$18,$5C,$0A,$D6,$19
;	1CC8
	.db	$84,$FD,$1A,$04,$6C,$19,$7C,$31
;	1CD0
	.db	$00,$E2,$90,$E2,$02,$72,$C2,$86
;	1CD8
	.db	$A0,$E6,$C2,$96,$C2,$34,$D6,$1A
;	1CE0
	.db	$4B,$30,$E8,$A6,$E7,$00,$EB,$05
;	1CE8
	.db	$46,$E3,$02,$8B,$08,$A6,$E7,$01
;	1CF0
	.db	$EB,$06,$46,$E3,$03,$8D,$1B,$80
;	1CF8
	.db	$A6,$E7,$02,$6B,$0A,$A6,$E7,$20
;	1D00
	.db	$6B,$05,$A6,$E7,$22,$EB,$05,$46
;	1D08
	.db	$E3,$04,$8B,$12,$A6,$E7,$03,$6B
;	1D10
	.db	$0A,$A6,$E7,$23,$6B,$05,$A6,$E7
;	1D18
	.db	$21,$EB,$09,$46,$E3,$05,$D6,$1B
;	1D20
	.db	$8E,$8D,$1B,$9F,$A6,$E7,$04,$6B
;	1D28
	.db	$05,$A6,$E7,$24,$EB,$05,$46,$E3
;	1D30
	.db	$06,$8B,$0E,$A6,$E7,$14,$6B,$06
;	1D38
	.db	$A6,$E7,$34,$ED,$1A,$04,$46,$E3
;	1D40
	.db	$07,$8C,$03,$8D,$1B,$9F,$A6,$E7
;	1D48
	.db	$07,$6D,$1C,$F5,$A6,$E7,$70,$ED
;	1D50
	.db	$1A,$04,$46,$E3,$10,$8B,$0F,$A6
;	1D58
	.db	$E7,$17,$6D,$1C,$F5,$A6,$E7,$71
;	1D60
	.db	$ED,$1A,$04,$46,$E3,$10,$D6,$1B
;	1D68
	.db	$8E,$8B,$8A,$A6,$E7,$48,$ED,$1A
;	1D70
	.db	$04,$8B,$1B,$A6,$E7,$E8,$6B,$07
;	1D78
	.db	$A6,$E7,$28,$EB,$07,$B0,$E5,$46
;	1D80
	.db	$E3,$02,$8B,$53,$A6,$E7,$38,$6B
;	1D88
	.db	$05,$A6,$E7,$78,$EB,$05,$8C,$02
;	1D90
	.db	$8D,$1B,$9F,$D6,$1E,$39,$46,$E3
;	1D98
	.db	$02,$8B,$A6,$A6,$E3,$0D,$EB,$25
;	1DA0
	.db	$A6,$E7,$E8,$6B,$07,$A6,$E7,$28
;	1DA8
	.db	$EB,$06,$B0,$E5,$3C,$8D,$8B,$27
;	1DB0
	.db	$A6,$E7,$78,$6B,$05,$A6,$E7,$38
;	1DB8
	.db	$EB,$04,$3C,$30,$8B,$D0,$D6,$1E
;	1DC0
	.db	$39,$3C,$8D,$8B,$0D,$A6,$E7,$E8
;	1DC8
	.db	$6B,$0D,$A6,$E7,$28,$6B,$08,$D6
;	1DD0
	.db	$1E,$39,$8C,$03,$8D,$1B,$9F,$48
;	1DD8
	.db	$E5,$58,$E6,$8C,$03,$8B,$02,$8C
;	1DE0
	.db	$01,$8D,$1B,$9F,$A6,$E7,$0E,$6B
;	1DE8
	.db	$06,$A6,$E7,$0F,$ED,$1A,$04,$56
;	1DF0
	.db	$E4,$0F,$F0,$E4,$42,$34,$48,$E5
;	1DF8
	.db	$58,$E6,$8B,$15,$A6,$E3,$0B,$EB
;	1E00
	.db	$02,$3C,$8B,$A6,$E7,$E8,$6B,$05
;	1E08
	.db	$D6,$1E,$39,$8B,$04,$48,$E5,$58
;	1E10
	.db	$E6,$F0,$2F,$EB,$20,$80,$E4,$80
;	1E18
	.db	$E4,$22,$5F,$32,$4E,$6B,$0E,$A6
;	1E20
	.db	$E4,$FF,$ED,$1A,$04,$A6,$E5,$80
;	1E28
	.db	$FB,$09,$8D,$1A,$04,$A6,$E5,$7F
;	1E30
	.db	$BD,$1A,$04,$48,$E5,$8C,$02,$8B
;	1E38
	.db	$A8,$A6,$E7,$F8,$ED,$1A,$04,$48
;	1E40
	.db	$E5,$58,$E6,RET_
	.db	$A6,$E7,$04,$EB
;	1E48
	.db	$06,$56,$E4,$CF,$8D,$1B,$97,$A6
;	1E50
	.db	$E7,$02,$6B,$05,$A6,$E7,$00,$EB
;	1E58
	.db	$06,$56,$E4,$8F,$8D,$1B,$97,$A6
;	1E60
	.db	$E7,$20,$EB,$0E,$56,$E5,$8F,$46
;	1E68
	.db	$E5,$10,$F0,$E5,$38,$E5,$8C,$02
;	1E70
	.db	$8B,$46,$A6,$E7,$05,$EB,$12,$3C
;	1E78
	.db	$C7,$56,$E4,$0F,$56,$E5,$0F,$F0
;	1E80
	.db	$E4,$42,$45,$58,$50,$8C,$03,$8B	;	.BEXP...
;	1E88
	.db	$2F,$A6,$E7,$50,$EB,$0C,$3C,$D7
;	1E90
	.db	$56,$E4,$0F,$56,$E5,$0F,$F0,$E5
;	1E98
	.db	$8B,$E7,$A6,$E7,$01,$EB,$05,$3C
;	1EA0
	.db	$E3,$8D,$1B,$80,$A6,$E7,$10,$EB
;	1EA8
	.db	$05,$3C,$F3,$8D,$1B,$80,$A6,$E7
;	1EB0
	.db	$22,$EB,$08,$3C,$E4,$D6,$1B,$8E
;	1EB8
	.db	$8D,$1B,$9F,$A6,$E7,$23,$6B,$0A
;	1EC0
	.db	$A6,$E7,$21,$6B,$05,$A6,$E7,$03
;	1EC8
	.db	$EB,$04,$3C,$E5,$8B,$E7,$A6,$E7
;	1ED0
	.db	$24,$EB,$04,$3C,$E6,$8B,$0C,$A6
;	1ED8
	.db	$E7,$34,$6B,$05,$A6,$E7,$14,$EB
;	1EE0
	.db	$06,$3C,$E7,$8C,$03,$8B,$D1,$A6
;	1EE8
	.db	$E7,$32,$6B,$0A,$A6,$E7,$12,$6B
;	1EF0
	.db	$05,$A6,$E7,$30,$EB,$36,$3C,$F5
;	1EF8
	.db	$8B,$BB,$A6,$E7,$28,$6B,$20,$A6
;	1F00
	.db	$E7,$08,$EB,$02,$8B,$19,$A6,$E7
;	1F08
	.db	$38,$6B,$05,$A6,$E7,$18,$EB,$1C
;	1F10
	.db	$46,$E3,$01,$8B,$0A,$A6,$E7,$28
;	1F18
	.db	$6B,$05,$A6,$E7,$68,$EB,$05,$8C
;	1F20
	.db	$02,$8D,$1B,$9F,$A6,$E7,$38,$6B
;	1F28
	.db	$06,$A6,$E7,$18,$ED,$1A,$04,$46
;	1F30
	.db	$E3,$01,$8B,$EB,$A6,$E7,$08,$EB
;	1F38
	.db	$C1,$F0,$E4,$38,$E4,$8C,$01,$8B
;	1F40
	.db	$E0,$A6,$E3,$02,$EB,$06,$B0,$EE
;	1F48
	.db	$B0,$EF,$8B,$45,$A6,$E3,$04,$EB
;	1F50
	.db	$40,$A6,$E7,$0A,$EB,$18,$D4,$2A
;	1F58
	.db	$A6,$E1,$00,$6D,$1B,$CD,$F0,$2F
;	1F60
	.db	$EB,$08,$04,$56,$EE,$D2,$1E,$24
;	1F68
	.db	$56,$EE,$A0,$EE,$8B,$E8,$A6,$E7
;	1F70
	.db	$0B,$EB,$40,$D4,$2A,$D6,$1A,$3F
;	1F78
	.db	$A6,$E1,$00,$6D,$1B,$CD,$D6,$19
;	1F80
	.db	$F9,$F0,$2F,$EB,$08,$04,$56,$EE
	.db	$D2,$5E,$24,$56,$EE,$A0,$EE,$8B
;	1F90
	.db	$E4,$A6,$E7,$28,$EB,$07,$02,$F5
	.db	$16,$EE,$00,$8B,$09,$A6,$E7,$E8
;	1FA0
	.db	$EB,$11,$02,$F6,$12,$E4,$8D,$1B
	.db	$CD,$F0,$2F,$6B,$09,$E4,$53,$51
;	1FB0
	.db	$E4,$54,$52,$8D,$1A,$04,$31,$30
	.db	$E6,$51,$FF,$E6,$52,$FF,$9C,$5E
;	1FC0
	.db	$8D,$1A,$08,$FF,$FF,$FF,$FF,$70
	.db	$FD,$31,$40,$FC,$05,$CC,$19,$DC
;	1FD0
	.db	$4F,$B0,$E2,$C2,$4C,$A0,$EC,$C2
	.db	$5C,$A0,$EC,$B0,$E3,$22,$15,$32
;	1FE0
	.db	$04,$3E,$FB,$F9,$02,$15,$12,$04
	.db	$00,$E3,$EB,$08,$52,$22,$EB,$04
;	1FF0
	.db	$3C,$20,$8B,$04,$2E,$06,$E3,$30


;	db	$D6,$17,$5E,$FA,$D6,$50,$FD,RET_
	.db	$D6,$17,$5E,$FA,$D6,$50,$FD,RET_

	.el

prog_00021:	;	$1984
	ld	r6,r10		;	1984  68 ea
	ld	r7,r11		;	1986  78 eb
	ld	r12,r1		;	1988  c8 e1
loop_198a:
	ld	r10,r6		;	198a  a8 e6
	ld	r11,r7		;	198c  b8 e7
	ld	r1,r12		;	198e  18 ec
loop_1990:
	ldc	r2,@rr4		;	1990  c2 24
	incw	rr4		;	1992  a0 e4
	cp	r2,#15		;	1994  a6 e2 0f
	jr	ugt,skip_0137	;	1997  bb 02
	scf			;	1999  df
	ret			;	199a  af

skip_0137:
	cp	r1,#65		;	199b  a6 e1 41
	jr	c,skip_0138	;	199e  7b 03
	and	r1,#223		;	19a0  56 e1 df
skip_0138:
	cp	r1,r2		;	19a3  a2 12
	jr	z,skip_0139	;	19a5  6b 18
	rcf			;	19a7  cf
loop_0140:
	ldc	r2,@rr4		;	19a8  c2 24
	incw	rr4		;	19aa  a0 e4
	cp	r2,#15		;	19ac  a6 e2 0f
	jr	ugt,loop_0140	;	19af  bb f7
	swap	r0		;	19b1  f0 e0
	jr	nz,skip_0141	;	19b3  eb 02
	incw	rr4		;	19b5  a0 e4
skip_0141:
	ldc	r2,@rr4		;	19b7  c2 24
	cp	r2,#255		;	19b9  a6 e2 ff
	jr	nz,loop_198a	;	19bc  eb cc
	ret			;	19be  af

skip_0139:
;;	call	@rr:42		;	19bf  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	jr	loop_1990	;	19c1  8b cd

prog_00017:			;	$19c3
	push	RP		;	19c3  70 fd
	ld	26,r10		;	19c5  a9 1a
	ld	27,r11		;	19c7  b9 1b
	cp	r1,#37		;	19c9  a6 e1 25
	ld	RP,46		;	19cc  e4 2e fd
	jr	nz,mark_0144	;	19cf  eb 0d
	ld	87,r11		;	19d1  b9 57
	call	BASIC_02f8	;	19d3  d6 02 f8
	cp	87,r11		;	19d6  a4 eb 57
	jp	z,func_1a04	;	19d9  6d 1a 04
	jr	mark_0146	;	19dc  8b 0b
mark_0144:
	decw	rr10		;	19de  80 ea
	call	BASIC_05d3	;	19e0  d6 05 d3
	jp	ge,func_1a04	;	19e3  9d 1a 04
	call	BASIC_02c4	;	19e6  d6 02 c4
mark_0146:
	call	BASIC_000e	;	19e9  d6 00 0e
	pop	RP		;	19ec  50 fd
	ld	r10,26		;	19ee  a8 1a
	ld	r11,27		;	19f0  b8 1b
	ld	r5,18		;	19f2  58 12
	ld	r6,19		;	19f4  68 13
;;	call	@rr:42		;	19f6  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	ret			;	19f8  af

prog_00023:			;	19f9
	call	prog_00017	;	19f9  d6 19 c3
	or	r5,r5		;	19fc  42 55
	jp	nz,func_1a04	;	19fe  ed 1a 04
	ld	r5,r6		;	1a01  58 e6
	ret			;	1a03  af

;
; This is often-called.
;
func_1a04:
	srp	#48		;	1a04  31 30

	ld	r9,#97		;	1a06  9c 61

mark_0270:			;	1a08
	ld	r8,#25		;	1a08  8c 19

	call	BASIC_0556	;	1a0a  d6 05 56

	ld	SPH,35		;	1a0d  e4 23 fe
	ld	SPL,36		;	1a10  e4 24 ff
	ld	RP,46		;	1a13  e4 2e fd
	call	prog_00007	;	1a16  d6 00 2b

	ld	18,81		;	1a19  e4 51 12
	ld	19,82		;	1a1c  e4 52 13

	ld	r10,33		;	1a1f  a8 21
	ld	r11,34		;	1a21  b8 22
	ret			;	1a23  af

prog_00019:			;	1a24
	cp	r1,#32		;	1a24  a6 e1 20
	jr	z,ret_1a3e	;	1a27  6b 15

	cp	r1,#44		;	1a29  a6 e1 2c
	jr	z,ret_1a3e	;	1a2c  6b 10

	cp	r1,#9		;	1a2e  a6 e1 09
	jr	z,ret_1a3e	;	1a31  6b 0b

	cp	r1,#33		;	1a33  a6 e1 21
	jr	z,scf_ret	;	1a36  6b 05

	cp	r1,#0		;	1a38  a6 e1 00
	jr	nz,ret_1a3e	;	1a3b  eb 01

scf_ret:
	scf			;	1a3d  df
ret_1a3e:
	ret			;	1a3e  af

prog_00020:			;	1a3f

loop_1a49:
	call	prog_00019	;	1a3f  d6 1a 24
	jr	nz,skip_1a46	;	1a42  eb 02
	jr	nc,skip_1a47	;	1a44  fb 01
skip_1a46:
	ret			;	1a46  af

skip_1a47:			;
;;	call	@rr:42		;	1a47  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	jr	loop_1a49	;	1a49  8b f4

prog_00032:

	call	prog_00020	;	1a4b  d6 1a 3f
	jr	nz,mark_0152	;	1a4e  eb 05
	jr	nc,mark_0152	;	1a50  fb 03
	ld	r7,#136		;	1a52  7c 88
	ret			;	1a54  af

mark_0152:			;
	cp	r1,#34		;	1a55  a6 e1 22
	jr	nz,mark_0153	;	1a58  eb 03
	ld	r7,#10		;	1a5a  7c 0a
	ret			;	1a5c  af

mark_0153:			;
	cp	r1,#38		;	1a5d  a6 e1 26
	jr	nz,mark_0154	;	1a60  eb 03
	ld	r7,#11		;	1a62  7c 0b
	ret			;	1a64  af

mark_0154:			;
	ld	r4,#24		;	1a65  4c 18
	ld	r5,#247		;	1a67  5c f7
	ld	r0,#255		;	1a69  0c ff
	call	prog_00021	;	1a6b  d6 19 84
	jr	nc,mark_0155	;	1a6e  fb 09
	swap	r2		;	1a70  f0 e2
	or	r3,r2		;	1a72  42 32
	call	prog_00020	;	1a74  d6 1a 3f
	jr	mark_0156	;	1a77  8b 06
mark_0155:			;
	ld	r10,r6		;	1a79  a8 e6
	ld	r11,r7		;	1a7b  b8 e7
	ld	r1,r12		;	1a7d  18 ec
mark_0156:			;
	clr	r7		;	1a7f  b0 e7
	call	prog_00022	;	1a81  d6 1a 97
	ld	r4,r5		;	1a84  48 e5
	swap	r7		;	1a86  f0 e7
	call	prog_00020	;	1a88  d6 1a 3f
	jr	nz,mark_0157	;	1a8b  eb 06
	jr	nc,mark_0157	;	1a8d  fb 04
	or	r7,#8		;	1a8f  46 e7 08
	ret			;	1a92  af

mark_0157:			;
	call	prog_00022	;	1a93  d6 1a 97
	ret			;	1a96  af

prog_00022:			;	1a97

	cp	r1,#82		;	1a97  a6 e1 52
	jr	z,mark_0158	;	1a9a  6b 05
	cp	r1,#114		;	1a9c  a6 e1 72
	jr	nz,mark_0159	;	1a9f  eb 21
mark_0158:			;
;;	call	@rr:42		;	1aa1  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	and	r7,#240		;	1aa3  56 e7 f0
	cp	r1,#82		;	1aa6  a6 e1 52
	jr	z,mark_0160	;	1aa9  6b 05
	cp	r1,#114		;	1aab  a6 e1 72
	jr	nz,mark_0161	;	1aae  eb 05
mark_0160:			;
;;	call	@rr:42		;	1ab0  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	or	r7,#6		;	1ab2  46 e7 06
mark_0161:			;
	call	prog_00023	;	1ab5  d6 19 f9
	cp	r5,#16		;	1ab8  a6 e5 10
	jp	nc,func_1a04	;	1abb  fd 1a 04
	or	r5,#224		;	1abe  46 e5 e0
	ret			;	1ac1  af

mark_0159:			;
	cp	r1,#64		;	1ac2  a6 e1 40
	jr	nz,mark_0162	;	1ac5  eb 29
;;	call	@rr:42		;	1ac7  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	cp	r1,#82		;	1ac9  a6 e1 52
	jr	z,mark_0163	;	1acc  6b 05
	cp	r1,#114		;	1ace  a6 e1 72
	jr	nz,mark_0164	;	1ad1  eb 16
mark_0163:			;
;;	call	@rr:42		;	1ad3  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	or	r7,#1		;	1ad5  46 e7 01
	cp	r1,#82		;	1ad8  a6 e1 52
	jr	z,mark_0165	;	1adb  6b 05
	cp	r1,#114		;	1add  a6 e1 72
	jr	nz,mark_0161	;	1ae0  eb d3
mark_0165:			;
	or	r7,#7		;	1ae2  46 e7 07
;	;	call	@rr:42	;	1ae5  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	jr	mark_0161	;	1ae7  8b cc
mark_0164:			;
	or	r7,#3		;	1ae9  46 e7 03
mark_0167:			;
	call	prog_00023	;	1aec  d6 19 f9
	ret			;	1aef  af

mark_0162:			;
	cp	r1,#35		;	1af0  a6 e1 23
	jr	nz,mark_0166	;	1af3  eb 07
;;	call	@rr:42		;	1af5  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	or	r7,#4		;	1af7  46 e7 04
	jr	mark_0167	;	1afa  8b f0
mark_0166:			;
	cp	r1,#42		;	1afc  a6 e1 2a
	jr	nz,mark_0168	;	1aff  eb 3c
	or	r7,#15		;	1b01  46 e7 0f
	swap	47		;	1b04  f0 2f
	jr	z,mark_0169	;	1b06  6b 07
mark_0170:			;
;;	call	@rr:42		;	1b08  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	swap	r1		;	1b0a  f0 e1
	jr	nz,mark_0170	;	1b0c  eb fa
	ret			;	1b0e  af

mark_0169:			;
;;	call	@rr:42		;	1b0f  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	call	prog_00017	;	1b11  d6 19 c3
	ld	44,r5		;	1b14  59 2c
	ld	45,r6		;	1b16  69 2d
	ld	47,#255		;	1b18  e6 2f ff
	srp	#64		;	1b1b  31 40
	ld	r1,39		;	1b1d  18 27
	ld	r10,40		;	1b1f  a8 28
	ld	r11,41		;	1b21  b8 29
	ld	r14,4		;	1b23  e8 04
	ld	r15,5		;	1b25  f8 05
	decw	rr10		;	1b27  80 ea
	ldc	r1,@rr10	;	1b29  c2 1a
	ld	83,81		;	1b2b  e4 51 53
	ld	84,82		;	1b2e  e4 52 54
	call	prog_00024	;	1b31  d6 1b e9
	ld	53,r14		;	1b34  e9 35
	ld	54,r15		;	1b36  f9 36
	clr	47		;	1b38  b0 2f
	srp	#48		;	1b3a  31 30
	ret			;	1b3c  af

mark_0168:			;
	call	prog_00017	;	1b3d  d6 19 c3
	cp	r1,#40		;	1b40  a6 e1 28
	jr	nz,mark_0171	;	1b43  eb 2c
	cp	r5,#0		;	1b45  a6 e5 00
	jp	nz,func_1a04	;	1b48  ed 1a 04
;;	call	@rr:42		;	1b4b  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	ld	80,r6		;	1b4d  69 50
	cp	r1,#82		;	1b4f  a6 e1 52
	jr	z,mark_0172	;	1b52  6b 06
	cp	r1,#114		;	1b54  a6 e1 72
	jp	nz,func_1a04	;	1b57  ed 1a 04
mark_0172:			;
	or	r7,#5		;	1b5a  46 e7 05
;;	call	@rr:42		;	1b5d  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	call	prog_00023	;	1b5f  d6 19 f9
	cp	r5,#16		;	1b62  a6 e5 10
	jp	nc,func_1a04	;	1b65  fd 1a 04
	cp	r1,#41		;	1b68  a6 e1 29
	jp	nz,func_1a04	;	1b6b  ed 1a 04
;;	call	@rr:42		;	1b6e  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above

	ret			;	1b70  af

mark_0171:			;
	cp	r5,#0		;	1b71  a6 e5 00
	jr	z,mark_0173	;	1b74  6b 04
	or	r7,#14		;	1b76  46 e7 0e
	ret			;	1b79  af

mark_0173:			;
	or	r7,#2		;	1b7a  46 e7 02
	ld	r5,r6		;	1b7d  58 e6
	ret			;	1b7f  af

mark_0199:			;	1b80
	and	r4,#15		;	1b80  56 e4 0f
	swap	r4		;	1b83  f0 e4
	and	r5,#15		;	1b85  56 e5 0f
	or	r4,r5		;	1b88  42 45
	ld	r8,#2		;	1b8a  8c 02
	jr	skip_1b9f	;	1b8c  8b 11

prog_00033:			;	1b8e
	ld	r6,r4		;	1b8e  68 e4
	ld	r4,r5		;	1b90  48 e5
	ld	r5,r6		;	1b92  58 e6
	ld	r8,#3		;	1b94  8c 03
	ret			;	1b96  af

mark_0232:			;	1b97
	swap	r4		;	1b97  f0 e4
	ld	r3,r4		;	1b99  38 e4
	ld	r4,r5		;	1b9b  48 e5
	ld	r8,#2		;	1b9d  8c 02


mark_0205:			;	1b9f
skip_1b9f:			;
	swap	47		;	1b9f  f0 2f
	jr	z,mark_0175	;	1ba1  6b 07
	add	r15,r8		;	1ba3  02 f8
	adc	r14,#0		;	1ba5  16 ee 00
	jr	mark_0176	;	1ba8  8b 23
mark_0175:			;
	ld	r2,#51		;	1baa  2c 33
	swap	85		;	1bac  f0 55
	jr	nz,mark_0177	;	1bae  eb 0a
	ld	r6,r14		;	1bb0  68 ee
	call	prog_00025	;	1bb2  d6 17 6d
	ld	r6,r15		;	1bb5  68 ef
	call	prog_00026	;	1bb7  d6 17 6a
mark_0177:			;
	ld	r6,@r2		;	1bba  e3 62
	add	r14,86		;	1bbc  04 56 ee
	ldci	@rr14,@r2	;	1bbf  d3 2e
	sub	r14,86		;	1bc1  24 56 ee
	swap	85		;	1bc4  f0 55
	jr	nz,mark_0178	;	1bc6  eb 03
	call	prog_00025	;	1bc8  d6 17 6d
mark_0178:			;
	djnz	r8,mark_0177	;	1bcb  8a ed

mark_0195:			;	1bcd
mark_0176:			;
	cp	r1,#0		;	1bcd  a6 e1 00
	jr	z,mark_0179	;	1bd0  6b 04
;;	call	@rr:42		;	1bd2  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	jr	mark_0176	;	1bd4  8b f7
mark_0179:			;
;;	call	@rr:42		;	1bd6  d4 2a
	.db	$d4,$2a		; sbasm does not assemble line above


	cp	r1,#255		;	1bd8  a6 e1 ff
	ld	81,r1		;	1bdb  19 51
	ldc	r1,@rr10	;	1bdd  c2 1a
	ld	82,r1		;	1bdf  19 52
	jr	nz,mark_0180	;	1be1  eb 06
	cp	r1,#255		;	1be3  a6 e1 ff
	jp	z,func_1a04	;	1be6  6d 1a 04


prog_00024:			;	1be9

mark_0180:			;
	swap	47		;	1be9  f0 2f
	jr	z,mark_0181	;	1beb  6b 11
	cp	45,r1		;	1bed  a4 e1 2d
	jr	nz,mark_0181	;	1bf0  eb 0c
	decw	rr10		;	1bf2  80 ea
	ldc	r1,@rr10	;	1bf4  c2 1a
	cp	44,r1		;	1bf6  a4 e1 2c
	jr	nz,mark_0182	;	1bf9  eb 01
	ret			;	1bfb  af

mark_0182:			;
	incw	rr10		;	1bfc  a0 ea
mark_0181:			;
	incw	rr10		;	1bfe  a0 ea
	jp	mark_0183	;	1c00  8d 1c 7c
	ld	81,20		;	1c03  e4 14 51
	ld	82,21		;	1c06  e4 15 52
	ld	42,#25		;	1c09  e6 2a 19
	ld	43,#89		;	1c0c  e6 2b 59
	ld	33,26		;	1c0f  e4 1a 21
	ld	34,27		;	1c12  e4 1b 22
	ld	46,RP		;	1c15  e4 fd 2e
	ld	35,SPH		;	1c18  e4 fe 23
	ld	36,SPL		;	1c1b  e4 ff 24
	srp	#48		;	1c1e  31 30
	clr	86		;	1c20  b0 56
	cp	18,20		;	1c22  a4 14 12
	jr	nz,mark_0184	;	1c25  eb 05
	cp	19,21		;	1c27  a4 15 13
	jr	z,mark_0185	;	1c2a  6b 03
mark_0184:			;
	ld	86,18		;	1c2c  e4 12 56
mark_0185:			;
	swap	85		;	1c2f  f0 55
	jr	nz,mark_0186	;	1c31  eb 0d
	call	prog_00027	;	1c33  d6 17 64
	ld	r8,#25		;	1c36  8c 19
	ld	r9,#103		;	1c38  9c 67
	call	BASIC_0556	;	1c3a  d6 05 56
	call	prog_00027	;	1c3d  d6 17 64
mark_0186:			;
	ld	r10,8		;	1c40  a8 08
	ld	r11,9		;	1c42  b8 09
	ld	r14,4		;	1c44  e8 04
	ld	r15,5		;	1c46  f8 05
	clr	r1		;	1c48  b0 e1
mark_0190:			;
	cp	r1,#0		;	1c4a  a6 e1 00
	jr	nz,mark_0187	;	1c4d  eb 0e
;;	call	@rr:42		;	1c4f  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	cp	r1,20		;	1c51  a4 14 e1
	jr	nz,mark_0187	;	1c54  eb 07
;;	call	@rr:42		;	1c56  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	cp	r1,21		;	1c58  a4 15 e1
	jr	z,mark_0188	;	1c5b  6b 11
mark_0187:			;
	cp	r1,#255		;	1c5d  a6 e1 ff
	jr	nz,mark_0189	;	1c60  eb 08
;;	call	@rr:42		;	1c62  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	cp	r1,#255		;	1c64  a6 e1 ff
	jp	z,func_1a04	;	1c67  6d 1a 04
mark_0189:			;
;;	call	@rr:42		;	1c6a  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	jr	mark_0190	;	1c6c  8b dc
mark_0188:			;
	ld	37,SPH		;	1c6e  e4 fe 25
	ld	38,SPL		;	1c71  e4 ff 26
	ld	39,r1		;	1c74  19 27
	ld	40,r10		;	1c76  a9 28
	ld	41,r11		;	1c78  b9 29
	clr	47		;	1c7a  b0 2f

mark_0183:			;	1c7c
	swap	47		;	1c7c  f0 2f
	jr	nz,mark_0191	;	1c7e  eb 38
	swap	85		;	1c80  f0 55
	jr	nz,mark_0192	;	1c82  eb 2c
	srp	#64		;	1c84  31 40
	call	prog_00028	;	1c86  d6 17 88
	jp	z,func_1a04	;	1c89  6d 1a 04
	call	prog_00027	;	1c8c  d6 17 64
	ld	r0,81		;	1c8f  08 51
	ld	r1,82		;	1c91  18 52
	call	prog_00029	;	1c93  d6 1f c7
	call	prog_00030	;	1c96  d6 17 61
	ld	r14,#25		;	1c99  ec 19
	ld	r4,58		;	1c9b  48 3a
	ld	r5,59		;	1c9d  58 3b
mark_0194:			;
	ldc	r3,@rr4		;	1c9f  c2 34
	and	r3,r3		;	1ca1  52 33
	jr	nz,mark_0193	;	1ca3  eb 04
	decw	rr4		;	1ca5  80 e4
	ld	r3,#32		;	1ca7  3c 20
mark_0193:			;
	incw	rr4		;	1ca9  a0 e4
	call	prog_00031	;	1cab  d6 17 5e
	djnz	r14,mark_0194	;	1cae  ea ef
mark_0192:			;
	srp	#48		;	1cb0  31 30
	ld	SPH,37		;	1cb2  e4 25 fe
	ld	SPL,38		;	1cb5  e4 26 ff
mark_0191:			;
;;	call	@rr:42		;	1cb8  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	clr	r0		;	1cba  b0 e0
	call	prog_00020	;	1cbc  d6 1a 3f
	jp	z,mark_0195	;	1cbf  6d 1b cd
	ld	r4,#24		;	1cc2  4c 18
	ld	r5,#10		;	1cc4  5c 0a
	call	prog_00021	;	1cc6  d6 19 84
	jp	nc,func_1a04	;	1cc9  fd 1a 04
	ld	r6,#25		;	1ccc  6c 19
	ld	r7,#49		;	1cce  7c 31
	dec	r2		;	1cd0  00 e2
	rl	r2		;	1cd2  90 e2
	add	r7,r2		;	1cd4  02 72
	ldc	r8,@rr6		;	1cd6  c2 86
	incw	rr6		;	1cd8  a0 e6
	ldc	r9,@rr6		;	1cda  c2 96
	ldc	r3,@rr4		;	1cdc  c2 34
	call	prog_00032	;	1cde  d6 1a 4b
	jp	@rr8		;	1ce1  30 e8
	cp	r7,#0		;	1ce3  a6 e7 00
	jr	nz,mark_0196	;	1ce6  eb 05
	or	r3,#2		;	1ce8  46 e3 02
	jr	mark_0197	;	1ceb  8b 08
mark_0196:			;
	cp	r7,#1		;	1ced  a6 e7 01
	jr	nz,mark_0198	;	1cf0  eb 06
	or	r3,#3		;	1cf2  46 e3 03


mark_0210:			;	1cf5
mark_0197:			;
	jp	mark_0199	;	1cf5  8d 1b 80
mark_0198:			;
	cp	r7,#2		;	1cf8  a6 e7 02
	jr	z,mark_0200	;	1cfb  6b 0a
	cp	r7,#32		;	1cfd  a6 e7 20
	jr	z,mark_0200	;	1d00  6b 05
	cp	r7,#34		;	1d02  a6 e7 22
	jr	nz,mark_0201	;	1d05  eb 05
mark_0200:			;
	or	r3,#4		;	1d07  46 e3 04
	jr	mark_0202	;	1d0a  8b 12
mark_0201:			;
	cp	r7,#3		;	1d0c  a6 e7 03
	jr	z,mark_0203	;	1d0f  6b 0a
	cp	r7,#35		;	1d11  a6 e7 23
	jr	z,mark_0203	;	1d14  6b 05
	cp	r7,#33		;	1d16  a6 e7 21
	jr	nz,mark_0204	;	1d19  eb 09
mark_0203:			;
	or	r3,#5		;	1d1b  46 e3 05
mark_0202:			;
	call	prog_00033	;	1d1e  d6 1b 8e
	jp	mark_0205	;	1d21  8d 1b 9f
mark_0204:			;
	cp	r7,#4		;	1d24  a6 e7 04
	jr	z,mark_0206	;	1d27  6b 05
	cp	r7,#36		;	1d29  a6 e7 24
	jr	nz,mark_0207	;	1d2c  eb 05
mark_0206:			;
	or	r3,#6		;	1d2e  46 e3 06
	jr	mark_0208	;	1d31  8b 0e
mark_0207:			;
	cp	r7,#20		;	1d33  a6 e7 14
	jr	z,mark_0209	;	1d36  6b 06
	cp	r7,#52		;	1d38  a6 e7 34
	jp	nz,func_1a04	;	1d3b  ed 1a 04
mark_0209:			;
	or	r3,#7		;	1d3e  46 e3 07
mark_0208:			;
	ld	r8,#3		;	1d41  8c 03
	jp	mark_0205	;	1d43  8d 1b 9f
	cp	r7,#7		;	1d46  a6 e7 07
	jp	z,mark_0210	;	1d49  6d 1c f5
	cp	r7,#112		;	1d4c  a6 e7 70
	jp	nz,func_1a04	;	1d4f  ed 1a 04
	or	r3,#16		;	1d52  46 e3 10
	jr	mark_0211	;	1d55  8b 0f
	cp	r7,#23		;	1d57  a6 e7 17
	jp	z,mark_0210	;	1d5a  6d 1c f5
	cp	r7,#113		;	1d5d  a6 e7 71
	jp	nz,func_1a04	;	1d60  ed 1a 04
	or	r3,#16		;	1d63  46 e3 10
mark_0211:			;
	call	prog_00033	;	1d66  d6 1b 8e
	jr	mark_0197	;	1d69  8b 8a
	cp	r7,#72		;	1d6b  a6 e7 48
	jp	nz,func_1a04	;	1d6e  ed 1a 04
	jr	mark_0212	;	1d71  8b 1b
	cp	r7,#232		;	1d73  a6 e7 e8
	jr	z,mark_0213	;	1d76  6b 07
	cp	r7,#40		;	1d78  a6 e7 28
	jr	nz,mark_0214	;	1d7b  eb 07
	clr	r5		;	1d7d  b0 e5
mark_0213:			;
	or	r3,#2		;	1d7f  46 e3 02
	jr	mark_0215	;	1d82  8b 53
mark_0214:			;
	cp	r7,#56		;	1d84  a6 e7 38
	jr	z,mark_0212	;	1d87  6b 05
	cp	r7,#120		;	1d89  a6 e7 78
	jr	nz,mark_0216	;	1d8c  eb 05
mark_0212:			;
	ld	r8,#2		;	1d8e  8c 02
	jp	mark_0205	;	1d90  8d 1b 9f
mark_0216:			;
	call	prog_00034	;	1d93  d6 1e 39
	or	r3,#2		;	1d96  46 e3 02
	jr	mark_0208	;	1d99  8b a6
	cp	r3,#13		;	1d9b  a6 e3 0d
	jr	nz,mark_0217	;	1d9e  eb 25
	cp	r7,#232		;	1da0  a6 e7 e8
	jr	z,mark_0218	;	1da3  6b 07
	cp	r7,#40		;	1da5  a6 e7 28
	jr	nz,mark_0219	;	1da8  eb 06
	clr	r5		;	1daa  b0 e5
mark_0218:			;
	ld	r3,#141		;	1dac  3c 8d
	jr	mark_0215	;	1dae  8b 27
mark_0219:			;
	cp	r7,#120		;	1db0  a6 e7 78
	jr	z,mark_0220	;	1db3  6b 05
	cp	r7,#56		;	1db5  a6 e7 38
	jr	nz,mark_0221	;	1db8  eb 04
mark_0220:			;
	ld	r3,#48		;	1dba  3c 30
	jr	mark_0212	;	1dbc  8b d0
mark_0221:			;
	call	prog_00034	;	1dbe  d6 1e 39
	ld	r3,#141		;	1dc1  3c 8d
	jr	mark_0222	;	1dc3  8b 0d
mark_0217:			;
	cp	r7,#232		;	1dc5  a6 e7 e8
	jr	z,mark_0215	;	1dc8  6b 0d
	cp	r7,#40		;	1dca  a6 e7 28
	jr	z,mark_0215	;	1dcd  6b 08
	call	prog_00034	;	1dcf  d6 1e 39
mark_0222:			;
	ld	r8,#3		;	1dd2  8c 03
	jp	mark_0205	;	1dd4  8d 1b 9f
mark_0215:			;
	ld	r4,r5		;	1dd7  48 e5
	ld	r5,r6		;	1dd9  58 e6
	ld	r8,#3		;	1ddb  8c 03
	jr	mark_0223	;	1ddd  8b 02
	ld	r8,#1		;	1ddf  8c 01
mark_0223:			;
	jp	mark_0205	;	1de1  8d 1b 9f
	cp	r7,#14		;	1de4  a6 e7 0e
	jr	z,mark_0224	;	1de7  6b 06
	cp	r7,#15		;	1de9  a6 e7 0f
	jp	nz,func_1a04	;	1dec  ed 1a 04
mark_0224:			;
	and	r4,#15		;	1def  56 e4 0f
	swap	r4		;	1df2  f0 e4
	or	r3,r4		;	1df4  42 34
	ld	r4,r5		;	1df6  48 e5
	ld	r5,r6		;	1df8  58 e6
	jr	mark_0225	;	1dfa  8b 15
	cp	r3,#11		;	1dfc  a6 e3 0b
	jr	nz,mark_0226	;	1dff  eb 02
	ld	r3,#139		;	1e01  3c 8b
mark_0226:			;
	cp	r7,#232		;	1e03  a6 e7 e8
	jr	z,mark_0227	;	1e06  6b 05
	call	prog_00034	;	1e08  d6 1e 39
	jr	mark_0225	;	1e0b  8b 04
mark_0227:			;
	ld	r4,r5		;	1e0d  48 e5
	ld	r5,r6		;	1e0f  58 e6
mark_0225:			;
	swap	47		;	1e11  f0 2f
	jr	nz,mark_0228	;	1e13  eb 20
	decw	rr4		;	1e15  80 e4
	decw	rr4		;	1e17  80 e4
	sub	r5,r15		;	1e19  22 5f
	sbc	r4,r14		;	1e1b  32 4e
	jr	z,mark_0229	;	1e1d  6b 0e
	cp	r4,#255		;	1e1f  a6 e4 ff
	jp	nz,func_1a04	;	1e22  ed 1a 04
	cp	r5,#128		;	1e25  a6 e5 80
	jr	nc,mark_0230	;	1e28  fb 09
	jp	func_1a04	;	1e2a  8d 1a 04
mark_0229:			;
	cp	r5,#127		;	1e2d  a6 e5 7f
	jp	ugt,func_1a04	;	1e30  bd 1a 04
mark_0230:			;
	ld	r4,r5		;	1e33  48 e5
mark_0228:			;
	ld	r8,#2		;	1e35  8c 02
	jr	mark_0223	;	1e37  8b a8


prog_00034:			;	1e39
	cp	r7,#248		;	1e39  a6 e7 f8
	jp	nz,func_1a04	;	1e3c  ed 1a 04
	ld	r4,r5		;	1e3f  48 e5
	ld	r5,r6		;	1e41  58 e6
	ret			;	1e43  af

	cp	r7,#4		;	1e44  a6 e7 04
	jr	nz,mark_0231	;	1e47  eb 06
	and	r4,#207		;	1e49  56 e4 cf
	jp	mark_0232	;	1e4c  8d 1b 97
mark_0231:			;
	cp	r7,#2		;	1e4f  a6 e7 02
	jr	z,mark_0233	;	1e52  6b 05
	cp	r7,#0		;	1e54  a6 e7 00
	jr	nz,mark_0234	;	1e57  eb 06
mark_0233:			;
	and	r4,#143		;	1e59  56 e4 8f
	jp	mark_0232	;	1e5c  8d 1b 97
mark_0234:			;
	cp	r7,#32		;	1e5f  a6 e7 20
	jr	nz,mark_0235	;	1e62  eb 0e
	and	r5,#143		;	1e64  56 e5 8f
	or	r5,#16		;	1e67  46 e5 10
	swap	r5		;	1e6a  f0 e5
	ld	r3,r5		;	1e6c  38 e5
	ld	r8,#2		;	1e6e  8c 02
	jr	mark_0236	;	1e70  8b 46
mark_0235:			;
	cp	r7,#5		;	1e72  a6 e7 05
	jr	nz,mark_0237	;	1e75  eb 12
	ld	r3,#199		;	1e77  3c c7
	and	r4,#15		;	1e79  56 e4 0f
	and	r5,#15		;	1e7c  56 e5 0f
	swap	r4		;	1e7f  f0 e4
mark_0239:			;
	or	r4,r5		;	1e81  42 45
	ld	r5,80		;	1e83  58 50
	ld	r8,#3		;	1e85  8c 03
	jr	mark_0236	;	1e87  8b 2f
mark_0237:			;
	cp	r7,#80		;	1e89  a6 e7 50
	jr	nz,mark_0238	;	1e8c  eb 0c
	ld	r3,#215		;	1e8e  3c d7
	and	r4,#15		;	1e90  56 e4 0f
	and	r5,#15		;	1e93  56 e5 0f
	swap	r5		;	1e96  f0 e5
	jr	mark_0239	;	1e98  8b e7
mark_0238:			;
	cp	r7,#1		;	1e9a  a6 e7 01
	jr	nz,mark_0240	;	1e9d  eb 05
	ld	r3,#227		;	1e9f  3c e3
	jp	mark_0199	;	1ea1  8d 1b 80
mark_0240:			;
	cp	r7,#16		;	1ea4  a6 e7 10
	jr	nz,mark_0241	;	1ea7  eb 05
	ld	r3,#243		;	1ea9  3c f3
	jp	mark_0199	;	1eab  8d 1b 80
mark_0241:			;
	cp	r7,#34		;	1eae  a6 e7 22
	jr	nz,mark_0242	;	1eb1  eb 08
	ld	r3,#228		;	1eb3  3c e4
mark_0245:			;
	call	prog_00033	;	1eb5  d6 1b 8e
mark_0236:			;
	jp	mark_0205	;	1eb8  8d 1b 9f
mark_0242:			;
	cp	r7,#35		;	1ebb  a6 e7 23
	jr	z,mark_0243	;	1ebe  6b 0a
	cp	r7,#33		;	1ec0  a6 e7 21
	jr	z,mark_0243	;	1ec3  6b 05
	cp	r7,#3		;	1ec5  a6 e7 03
	jr	nz,mark_0244	;	1ec8  eb 04
mark_0243:			;
	ld	r3,#229		;	1eca  3c e5
	jr	mark_0245	;	1ecc  8b e7
mark_0244:			;
	cp	r7,#36		;	1ece  a6 e7 24
	jr	nz,mark_0246	;	1ed1  eb 04
	ld	r3,#230		;	1ed3  3c e6
	jr	mark_0247	;	1ed5  8b 0c
mark_0246:			;
	cp	r7,#52		;	1ed7  a6 e7 34
	jr	z,mark_0248	;	1eda  6b 05
	cp	r7,#20		;	1edc  a6 e7 14
	jr	nz,mark_0249	;	1edf  eb 06
mark_0248:			;
	ld	r3,#231		;	1ee1  3c e7
mark_0247:			;
	ld	r8,#3		;	1ee3  8c 03
	jr	mark_0236	;	1ee5  8b d1
mark_0249:			;
	cp	r7,#50		;	1ee7  a6 e7 32
	jr	z,mark_0250	;	1eea  6b 0a
	cp	r7,#18		;	1eec  a6 e7 12
	jr	z,mark_0250	;	1eef  6b 05
	cp	r7,#48		;	1ef1  a6 e7 30
	jr	nz,mark_0251	;	1ef4  eb 36
mark_0250:			;
	ld	r3,#245		;	1ef6  3c f5
	jr	mark_0245	;	1ef8  8b bb
mark_0257:			;
	cp	r7,#40		;	1efa  a6 e7 28
	jr	z,mark_0252	;	1efd  6b 20
	cp	r7,#8		;	1eff  a6 e7 08
	jr	nz,mark_0253	;	1f02  eb 02
	jr	mark_0252	;	1f04  8b 19
mark_0253:			;
	cp	r7,#56		;	1f06  a6 e7 38
	jr	z,mark_0254	;	1f09  6b 05
	cp	r7,#24		;	1f0b  a6 e7 18
	jr	nz,mark_0251	;	1f0e  eb 1c
mark_0254:			;
	or	r3,#1		;	1f10  46 e3 01
	jr	mark_0252	;	1f13  8b 0a
	cp	r7,#40		;	1f15  a6 e7 28
	jr	z,mark_0252	;	1f18  6b 05
	cp	r7,#104		;	1f1a  a6 e7 68
	jr	nz,mark_0255	;	1f1d  eb 05
mark_0252:			;
	ld	r8,#2		;	1f1f  8c 02
mark_0258:			;
	jp	mark_0205	;	1f21  8d 1b 9f
mark_0255:			;
	cp	r7,#56		;	1f24  a6 e7 38
	jr	z,mark_0256	;	1f27  6b 06
	cp	r7,#24		;	1f29  a6 e7 18
mark_0251:			;
	jp	nz,func_1a04	;	1f2c  ed 1a 04
mark_0256:			;
	or	r3,#1		;	1f2f  46 e3 01
	jr	mark_0252	;	1f32  8b eb
	cp	r7,#8		;	1f34  a6 e7 08
	jr	nz,mark_0257	;	1f37  eb c1
	swap	r4		;	1f39  f0 e4
	ld	r3,r4		;	1f3b  38 e4
	ld	r8,#1		;	1f3d  8c 01
	jr	mark_0258	;	1f3f  8b e0
	cp	r3,#2		;	1f41  a6 e3 02
	jr	nz,mark_0259	;	1f44  eb 06
	clr	r14		;	1f46  b0 ee
	clr	r15		;	1f48  b0 ef
	jr	mark_0260	;	1f4a  8b 45
mark_0259:			;
	cp	r3,#4		;	1f4c  a6 e3 04
	jr	nz,mark_0260	;	1f4f  eb 40
	cp	r7,#10		;	1f51  a6 e7 0a
	jr	nz,mark_0261	;	1f54  eb 18
mark_0263:			;
;;	call	@rr:42		;	1f56  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


	cp	r1,#0		;	1f58  a6 e1 00
	jp	z,mark_0195	;	1f5b  6d 1b cd
	swap	47		;	1f5e  f0 2f
	jr	nz,mark_0262	;	1f60  eb 08
	add	r14,86		;	1f62  04 56 ee
	ldc	@rr14,r1	;	1f65  d2 1e
	sub	r14,86		;	1f67  24 56 ee
mark_0262:			;
	incw	rr14		;	1f6a  a0 ee
	jr	mark_0263	;	1f6c  8b e8
mark_0261:			;
	cp	r7,#11		;	1f6e  a6 e7 0b
	jr	nz,mark_0264	;	1f71  eb 40
;;	call	@rr:42		;	1f73  d4 2a
	.db	$d4,$2a	; sbasm does not assemble line above


mark_0266:			;
	call	prog_00020	;	1f75  d6 1a 3f
	cp	r1,#0		;	1f78  a6 e1 00
	jp	z,mark_0195	;	1f7b  6d 1b cd
	call	prog_00023	;	1f7e  d6 19 f9
	swap	47		;	1f81  f0 2f
	jr	nz,mark_0265	;	1f83  eb 08
	add	r14,86		;	1f85  04 56 ee
	ldc	@rr14,r5	;	1f88  d2 5e
	sub	r14,86		;	1f8a  24 56 ee
mark_0265:			;
	incw	rr14		;	1f8d  a0 ee
	jr	mark_0266	;	1f8f  8b e4
mark_0260:			;
	cp	r7,#40		;	1f91  a6 e7 28
	jr	nz,mark_0267	;	1f94  eb 07
	add	r15,r5		;	1f96  02 f5
	adc	r14,#0		;	1f98  16 ee 00
	jr	mark_0268	;	1f9b  8b 09
mark_0267:			;
	cp	r7,#232		;	1f9d  a6 e7 e8
	jr	nz,mark_0264	;	1fa0  eb 11
	add	r15,r6		;	1fa2  02 f6
	adc	r14,r4		;	1fa4  12 e4
mark_0268:			;
	jp	mark_0195	;	1fa6  8d 1b cd
	swap	47		;	1fa9  f0 2f
	jr	z,mark_0269	;	1fab  6b 09
	ld	81,83		;	1fad  e4 53 51
	ld	82,84		;	1fb0  e4 54 52
mark_0264:			;
	jp	func_1a04	;	1fb3  8d 1a 04
mark_0269:			;
	srp	#48		;	1fb6  31 30
	ld	81,#255		;	1fb8  e6 51 ff
	ld	82,#255		;	1fbb  e6 52 ff
	ld	r9,#94		;	1fbe  9c 5e
	jp	mark_0270	;	1fc0  8d 1a 08
	nop			;	1fc3  ff
	nop			;	1fc4  ff
	nop			;	1fc5  ff
	nop			;	1fc6  ff


prog_00029:			;	1fc7
	push	RP		;	1fc7  70 fd
	srp	#64		;	1fc9  31 40
	ld	r15,#5		;	1fcb  fc 05
	ld	r12,#25		;	1fcd  cc 19
	ld	r13,#79		;	1fcf  dc 4f
	clr	r2		;	1fd1  b0 e2
mark_0274:			;
	ldc	r4,@rr12	;	1fd3  c2 4c
	incw	rr12		;	1fd5  a0 ec
	ldc	r5,@rr12	;	1fd7  c2 5c
	incw	rr12		;	1fd9  a0 ec
	clr	r3		;	1fdb  b0 e3
mark_0271:			;
	sub	r1,r5		;	1fdd  22 15
	sbc	r0,r4		;	1fdf  32 04
	inc	r3		;	1fe1  3e
	jr	nc,mark_0271	;	1fe2  fb f9
	add	r1,r5		;	1fe4  02 15
	adc	r0,r4		;	1fe6  12 04
	dec	r3		;	1fe8  00 e3
	jr	nz,mark_0272	;	1fea  eb 08
	and	r2,r2		;	1fec  52 22
	jr	nz,mark_0272	;	1fee  eb 04
	ld	r3,#32		;	1ff0  3c 20
	jr	mark_0273	;	1ff2  8b 04
mark_0272:			;
	inc	r2		;	1ff4  2e
	add	r3,#48		;	1ff5  06 e3 30
mark_0273:			;
	call	prog_00031	;	1ff8  d6 17 5e
	djnz	r15,mark_0274	;	1ffb  fa d6
	pop	RP		;	1ffd  50 fd
	ret			;	1fff  af

	.fi

	.en
